*************************************************
************* Introduction to Stata *************
* 
* Chelsea Goforth
* Statistical Consulting Associate
* StatLab: Statistics and Data Analysis
* University of Virginia
* cag5fk@virginia.edu
* statlab@virginia.edu
* http://data.library.virginia.edu/statlab/
*
*************************************************


/* *****
I am indebted to Michele Claibourn (StatLab, UVa), Clay Ford (StatLab, UVa), and Jon 
Kropko (Politics, UVa) for many useful ideas and examples used throughout this presentation.
***** */


**************************
*** STARTING A DO-FILE ***
**************************

/* 
First, some do-file tips:

Useful Shortcuts: 
	ctrl+9 --> opens a new do-file editor 
	ctrl+N --> opens a new do-file
	ctrl+D --> to submit commands (or click the 'execute' button) 

Syntax Highlighting: 
	blue --> denotes built-in commands/functions
	red --> denotes string text (not commands/functions or other non-numeric information) 
	green --> denotes comments (information not processed by Stata when executed) 
	
	Note: can change the default syntax highlighting through "Edit --> Preferences"

Using comments to annotate your do file (three ways):
	1)  use an asterisk (*) at the beginning of a line (single line comment)
	2)  use two forward slashes (//) at the end of any command line (single line comment)
	3)  use two forward slashes and two asterisks (/* */) to bookend your comment (multi-line
		comment such that anything in between the two asterisks will constitute a comment)
	
	Note that I have used examples of all three types of comments throughout this do-file!

Finally, three forward slashes (///) will break up any long command into multiple lines.
*/


*** A standard do-file preamble:

clear // clears out any existing data from Stata's working memory

version 14 // specifies the version of Stata you're using

/* Stata periodically releases new versions, which means do-files written for older 
versions might stop working.  The above command prevents that from happening. */

set more off // tells Stata to continue issuing commands, even if the results window is full

/* Every time the results window fills up, Stata stops, even if in the middle of a calculation, 
and writes -more- at the bottom of the screen; you have to click to see further output.  This is 
very annoying, but if you use -set more off- you can let Stata run while you do other things.  
Note, though, that this is a temporary command.  It must be run within a block of commands from a 
do-file for more to be set off. If you are working in the command window or if you run something 
in the middle of a do-file, -more- will still appear. */

cd "" // changes the directory to whatever folder is specified
cd "C:\Users\Chelsea\Dropbox\U-VIRGINIA\StatLab\Workshops\6_Fall 2015 StatLab Workshops\1_Intro to Stata\working file"

/* The above command is specific to my computer.  Choose a different folder on your computer and 
specify the pathway in between the quotation marks.  Be aware that this is one of the few Stata 
features that may depend on platform differences between Windows and Mac OS X because of its 
relationship to the native operating system commands, e.g., using backslashes in the directory path 
on Windows (as above) vs. something like: "/Users/Goforth/Documents/introStata" on a Mac. */

/* You can also change the directory through "File --> Change Working Directory" and the 
current directory is always displayed in the lower left corner of the Stata interface.  This will 
indicate the folder on your computer to which any files created in your Stata session will be 
saved, including log files, graphics, tables, etc.  If you *don't* want a file saved to this 
directory, you either need to change it or specify the full pathway when saving (as opposed to 
simply the file name).  I never use the default "Documents" directory; instead, I create a new 
folder for each project or task and change the directory accordingly each time. */

capture log close // closes any log file that may have already been open

/* Note that -log close- often also works to this end; however, Stata will get stuck and issue 
an error message if there isn't a log open to close.  Adding -capture- to that command tells 
Stata to close a log if one is open, but to ignore the command and move on if not. */

log using "introStata.log", replace 

/* Logs allow you to document a full record of your Stata session, including all commands 
issued as well as all output generated. They can be recorded in two different formats; if no 
file extension is specified, the default is .smcl (Stata Markup and Control Language -- preserves 
fonts and colors), but I prefer saving as plain text (by specifying either .log or .txt). You can
also use the -translate- command to convert a .smcl log into text or other formats. */

/* There are two options you might find particularly useful when using logs: the -replace- option 
as in the above command will overwrite any log in the current directory that has the same name, 
or, you might want to use the -append- option to add to that log.  You can also use the commands
-log off- and -log on- to temporarily suspend writing to your log file without fully closing it. */

**********************************
*** READING IN AND SAVING DATA ***
**********************************

*** Delimited Text Data Files (aka "ASCII" Files): 

/* Use this option for ASCII data files, or, text files in which data points are usually delimited 
by commas, spaces, or tabs.  These files are designed to be as portable as possible, so they are not 
dependent on any partcular software, but they lack variable/value labels, and they may require a 
codebook and/or data dictionary to read in. */

/* Notes for creating ASCII files to read into Stata:
	1) The first line should have variable names, which need to be 32 characters or fewer; spaces 
	and special characters [except the underscore (_)] will be removed.  Stata is case-sensitive, 
	so it's good practice to use all lower-case letters in variable names.  And, you can't start 
	a variable name with a number (technically, you can start one with an underscore, but this is not 
	good practice because Stata's built-in variables begin with an underscore.
	
	2) No blank rows or columns between data.
	
	3) Missing numeric data should be coded as an empty cell, not a space, dot, or any other 
	non-numeric data. Often, 0, 9, or 99 is used to code missing numeric data; this is fine as long 
	as these are not also valid values (we can recode them to missing once in Stata).
	
	4) Commas in numbers or text are problematic, Stata thinks they are a delimiter and won't read 
	the data properly. You must remove the commas from numeric values before saving the file.
*/

import delimited "collegeCompletion.txt", clear 
* Stata is usually pretty good at figuring out how to automatically read in these data, but...

import delimited "collegeCompletion.txt", delimiter(tab) varnames(1) clear
* ... you can also specify options to make sure it's reading in exactly how you want it to.

/* Options for -import delimited- :
	-delimiter(tab)- specifies what character is separating the data points in this file
	-varnames(1)- tells Stata that the first row contains variable names
	-clear- removes data currently in memory (can only have one dataset open at a time)
*/

* Ditto for other kinds of delimited files like .csv (comma-separated values):
import delimited "collegeCompletion.csv", clear
import delimited "collegeCompletion.csv", delimiter(comma) varnames(1) clear

* For Stata 12 or earlier versions:
insheet using "collegeCompletion.txt", clear 

*** Saving:
save "collegeCompletion.dta", replace // again, replace overwrites the file if it exists

*** Excel Data Files: 

import excel "collegeCompletion.xlsx", sheet("collegeCompletionInstitutionDet") firstrow clear

/* Options for -import excel- :
	-sheet("")- specifies which sheet in the Excel workbook to import
	-firstrow- tells Stata that the first row contains variable names
	
	Note that -import excel- is only available in Stata 12 or later.
*/

*** Stata Data Files: 

use "collegeCompletion", clear // .dta file extension is the default here

**********************************
*** DATA MANAGEMENT & CLEANING ***
**********************************

*** Examining & Sorting Data:

describe 

browse // read-only spreadsheet
edit // can enter/edit data by hand, but I STRONGLY advise against this

/* Note that a period (.) indicates missing data.  Stata treats these as very large 
numbers, so be careful when using logical operators like greater than (>). */ 

* -list- to look at data in the results menu:
list chronname state // lists selected variables

/* That returned a lot of results! 

Use: 
	enter -- to advance one line at a time
	spacebar -- to advance one page at a time
	q (or break button) -- to quit

When commands are submitted in batch via a do-file, this is what -set more off- gets rid of.
*/

list chronname state level control in 1/10 // list selected variables for observations 1 to 10

* -if- to subset data (using any combination of relational and logical operators):
list chronname state level control if flagship=="X"
list chronname state control if grad_100_value==0 & level=="4-year"

/* How many institutions in Virginia have an on-time graduation rate of 0? */

* Add notes to data:
notes: This data was downloaded from http://collegecompletion.chronicle.com/
describe
notes

* -sort- to sort data in ascending order: 
sort unitid // sorting by initial id
sort student_count // by number of students

list chronname state control level student_count in 1/10 // lists 10 schools with fewest students

* -gsort- to sort data in descending order: 
gsort -aid_value // add - to variable to indicate descending

list chronname state control level aid_value in 1/10 // lists top 10 in aid value per student

*** Summary Statistics and Frequencies:

* -summarize- to look at means, standard deviations, min and max values:
summ grad_150_value // note the abbreviated command
summ grad_150_value grad_100_value student_count fte_value med_sat_value endow_value ft_fac_value
summ endow_value if control=="Public" & level=="4-year"

/* Which public universities have the biggest endowments/student? */
list chronname endow_value if contro=="Public" & level=="4-year" & endow_value>100000 & endow_value!=.

* -tabulate- for frequency distributions:
tabulate control
tab level

tab1 control level // -tab1- for sequential frequency tables for multiple variables

tab control level // cross-tabs to view the relationship between two categorical/ordered variables

tab control level, col // same as above, but includes the column percentage option

*** Variable Manipulation/Generation: 

* -encode- to change a variable from string to numeric:
encode control, gen(controlcat)
br control controlcat

* -rename- to rename variables:
rename grad_150_value grad150
rename grad_100_value grad100

/* Best practices for variable names: keep them short, descriptive, and reasonably 
distinguished from other variable names. If you have a set of variables that are 
related to each other (e.g., part of a scale), consider using the same prefix or 
suffix for all that are related. */

* -recode- to recode variables:
recode grad100 (0/33.3=1) (33.4/66.7=2) (66.8/100=3), gen(gradrate100)
recode grad100 (0/33.3=1) (33.4/66.7=2) (66.8/100=3), into(gradrate150)

/* Note that the -generate()- (or -gen()- for short) option preserves the original variable 
and recodes it into a new variable, which is named whatever you specify in between the 
parentheses; -into()- is a synonym and does exactly the same thing. Without one of these 
two options, the variables will be recoded so as to overwrite the originals. */

tab1 gradrate100 gradrate150

* -generate- and -egen- to generate new variables:
generate id = _n // generate an id number

gen lnaid = ln(aid_value) // as a mathematical function of an existing variable
summ aid_value lnaid

/* See -help math functions- for more operations that you can perform. */

gen highgradrate = (gradrate100==3) // as a logical function of an existing variable
tab1 gradrate100 highgradrate

/* Notice the difference between using one equals sign (=) for arithmetic 
operations and two (==) for logical operations (which often follow -if-). */

tab basic
gen research = 1 if basic=="Research Universities--high research activity" | /// 
	basic=="Research Universities--very high research activity" | ///
	basic=="Doctoral and Research Universities" 
replace research=0 if research==.
tab research

/* Using three forward slashes is a useful way to break up long commands; three slashes 
tells Stata that the command continues on the next line (not to be confused with two forward 
slahses, which denotes a comment, as previously discussed). */

gen flagshipdum=1 if flagship=="X" // another as a logical function of an existing variable
replace flagshipdum=0 if flagship==""
tab1 flagship flagshipdum

/* -egen- is used for more sophisticated transformations ('EGEN' = 'Extensions to GENerate') */

egen stdaid = std(aid_value) // standardized values of aid_value

* -drop- and -keep- to drop variables
drop vsa_* // the asterisk means "all" -- i.e., drop all variables that start with "vsa_"

/* Since the "vsa_" variables are all next to each other in the data file, I could have also 
written the command as: -drop vsa_year-vsa_enroll_elsewhere_after6_tran- (where vsa_year is 
the first variable, vsa_enroll_elsewhere_after6_tran is the last variable, and the middle dash 
captures every variable in between). */

/* -keep- works the same as -drop- except you tell Stata to drop everything 
EXCEPT the variables specified. */

*** Variable and Value Labels:

* -label variable- to add variable labels:
label variable research "Carnegie Classification-Research"
la var grad100 "Percent of first-time, full-time undergrads completing degree on time" 
la var level "2-year or 4-year institution"
la var control "Control of Institution"
describe research grad100

/* -label variable varname ""- to drop variable label (i.e., you're re-assigning it a blank label) */

* -label define- and -label values- to add value labels:
label define gradrate 1 "less than one-third" 2 "one-third to two-thirds" 3 "more than two-thirds"
label values gradrate100 gradrate150 gradrate
tab1 gradrate100 gradrate150

/* -label drop- will remove value labels */

la var flagshipdum "State Flagship"
la de yn 1 "yes" 0 "no"
la val flagshipdum yn
describe flagshipdum
tab flagshipdum

/* Once variable labels are applied, you can use the option -nolabel- with a variety of 
commands in order to display numerical values instead of words in the output. */

* -label data- to label the data set
label data "College Completion Data obtained from The Chronical of Higher Education"
describe 

save collegeCompletion_analysis.dta, replace 

/* Notice that I'm preserving the original data by saving my manipulated data using a 
different name. Also in the interest of good workflow practices, I'm using the logic 
of dual workflow to separate data management from analysis. I start my analyses in the 
next section by reading in my cleaned data (for larger projects, these might be two or 
more separate do-files). */

*************************************************
*** DESCRIPTIVE STATS & INTRODUCTORY ANALYSES ***
*************************************************

use collegeCompletion_analysis.dta, replace

*** Review: -tabulate- and -summarize- (with expanded options/details)

* -summarize- for continuous variables 
summ aid_value
summ grad150, detail

bysort control: summ aid_value // summarize over subsets
bysort level: summ grad100

summ aid_value
return list // see stored values in memory, whih can be used in subsequent calculations
display r(mean)
gen aid_center = aid_value - r(mean) // creates variable of centered values

* -tabulate- for frequency distributions of categorical variables
tab flagshipdum level, col chi2 // -chi2- option for a chi-squared statistic and p-value

tabulate level, summ(grad100) // one- and two-way tables of summary statistics

/* Compare this example to the command above: -bysort level: summ grad100- ... */

* other -tab- examples:

/*  What's the on-time graduation rate among instituions of different control types? */
tab gradrate100 control, col

/* Do graduation rates vary by private/public control among 4-year, among 2-year institutions? */
bysort level: tab gradrate100 control, col chi2

/* Among RESEARCH universities, do graduation rates vary by private/public control? */
tab gradrate150 control if research==1, col chi2

*** Using -table- to create custom tables:

/* The -table- command is especially useful for creating custom tables. The general 
syntax is as follows:
-table rowvar colvar, contents(stat1 var1 stat2 var2)-
where stat1 stat2 refer to mean, sd, min, max, etc.
Note that both the column variable and the second statistics (stat2 var2) are optional.
*/

/* What is the average number of undergrads and average percent of undergrads receiving 
Pell grants at each level of graduation rate and each category of control? */ 
table gradrate150 control, contents(mean student_count mean pell_value)

*** T-tests:
ttest grad100=grad150 // test difference of means between two continuous variables

ttest grad100, by(flagshipdum) // test the difference in the mean of one variable within categories of another

*** ANOVA:

* -oneway- for oneway anovas:
oneway grad150 control if research==1
oneway grad150 control if research==1, tab bon // use the original numeric version of the outcome
oneway grad150 control if research==1 & controlcat!=1, tab bon

/* The -tab- option produces a summary table of descriptive statistics, and the -bon- option 
(short for Bonferroni) will compute a multiple comparisons test, which calculates pairwise 
comparisons and then adjusts them for multiple comparisons */

*** Regression:

regress grad150 i.control if research==1 // can't include string variables
regress grad150 i.controlcat if research==1 // -i.- tells stata this is a categorical variable
regress grad150 i.controlcat if research==1 & controlcat!=1
regress grad150 i.controlcat c.med_sat_value if research==1 // -c.- indicates a continuous variable
regress grad150 i.controlcat c.med_sat_value i.controlcat#c.med_sat_value if research==1 // the pound sign means interaction

*****************************
*** INTRODUCTORY GRAPHICS ***
*****************************

/* Stata has powerful graphical facilities. As a consequence the code for creating 
nice graphs can get complicated. We'll be focused on just the basics today, but for 
more on graphics in Stata, download previous StatLab materials from Spring 2015. */

* -histogram- to view the distribution of continuous or categorical variables
hist grad150
hist pell_value, frac normal // notice the first graph has been replaced
hist student_count, freq // who's got 250k students?
list chronname student_count if student_count>100000

* use the -name- option to keep multiple graphs open:
hist lnaid, name(aid, replace)
hist pell_value, name(pell, replace)

* -set autotabgraphs on- to have graphs open in multiple tabs:
set autotabgraphs on
hist lnaid, name(aid, replace)
hist pell_value, name(pell, replace)

* -graph dir- to see which graphs are stored in memory
graph dir

* -graph display- to call them back to a viewer window after you've closed them:
graph display aid

* -graph drop- to drop graphs from memory:
graph drop aid
graph drop _all // drops everything

* -graph box- to create box plots to compare the distribution of an interval var across levels of a categorical var
graph box endow_value, over(control) // -over- option segments the plot by the values of the listed variable
graph box endow_value, over(control) nofill // -nofill- option omits the empty category
gen lnendow=ln(endow_value) // log the outcome variable
graph box lnendow if level=="4-year", over(control) nofill // subset with -if-

* -graph twoway scatter- to create a scatter plot to view the relationship between two interval variables
graph twoway scatter grad150 pell_value if level=="4-year" & control=="Public"
twoway scatter grad150 med_sat_value if level=="4-year" & control=="Public" 
scatter grad150 aid_value if level=="4-year" & control=="Public"  // notice the command abbreviations

/* Among 4-year research universities, do graduation rates vary by average student aid? */
twoway scatter grad150 lnaid if research==1 & controlcat!=1 

/* Making graphics look prettier often requires a lot of options, resulting in some complicated-looking 
code.  It's not as scary as it seems!  But for more explanation, check out the materials from my Intro 
to Stata Graphics workshop from Spring 2015. */

* -lfit- to add linear regression line; -lowess- for a smoothed locally weighted line:
twoway scatter grad150 lnaid || lfit grad150 lnaid || lowess grad150 lnaid || if research==1 & controlcat!=1 

* -legend- option (with everything in partentheses as sub-options) for more control over the legend
	* -order(2 3)- omits the first item in the legend
	* -label(# "")- specifies a better label for each relevant item
* -title- to add graph title 
* -xtitle- and -ytitle- options to add x-axis and y-axis titles
twoway scatter grad150 lnaid || lfit grad150 lnaid || lowess grad150 lnaid || if research==1 & controlcat!=1, ///
   legend(order(2 3) label(2 "Regression Line") label(3 "Lowess Line")) title("Completion Rate by Student Aid") ///
   xtitle("Average Student Aid (logged)") ytitle("Completion within 150% of Expected Time")

/* Does the relationship look the same for public/private universities? */
twoway scatter grad150 lnaid if controlcat==2, msymbol(Oh) mcolor(green) || ///
	scatter grad150 lnaid if controlcat==3, msymbol(Oh) mcolor(blue) || ///
	lfit grad150 lnaid if controlcat==2, lcolor(green) || lfit grad150 lnaid if controlcat==3, lcolor(blue) || ///
	lowess grad150 lnaid if controlcat==2, lcolor(midgreen) || lowess grad150 lnaid if controlcat==3, lcolor(midblue) || ///
	if research==1, ytitle("Completion Rate") xtitle("Average Student Aid (logged)") ///
	title("6-Year Completion Rates by Average Student Aid") subtitle("Across 4-Year Research Universities") ///
	legend(label(1 "Private") label(2 "Public") label(3 "OLS Fit") label(4 "OLS Fit") label(5 "Lowess Curve") label(6 "Lowess Curve"))

log close

save collegeCompletion_analysis_cg.dta, replace 

